import * as vscode from 'vscode';
import * as child_process from 'child_process';

const GitFilter:{[key: string]: string} = {
    s: 'since',
    u: 'until',
    a: 'author',
    g: 'grep',
};
export interface GitCommit {
    hash?: string;
    author?: string;
    date?: string;
    msg?: string;
    files?: string[];
}

export class GitService {
    private _repoPath: string;

    constructor() {
        const workspaceFolders = vscode.workspace.workspaceFolders;
        if (!workspaceFolders || workspaceFolders.length === 0) {
            throw new Error('No workspace folder opened');
        }
        this._repoPath = workspaceFolders[0].uri.fsPath;
    }

    public async getCommits(filter: string = ''): Promise<any[]> {
        const filterParts = filter.split(' ');
        const nFilter: string[] = [];
        filterParts.forEach((part) => {
            const t = part.split(':');
            const key = t[0];
            const v = t[1];
            if (key in GitFilter) {
                nFilter.push(`--${GitFilter[key]}=${v}`);
            }
        });
        const command = `git log --oneline ${nFilter.join(' ')} --pretty=format:"{\\\"hash\\\":\\\"%h\\\",\\\"author\\\":\\\"%an\\\",\\\"date\\\":\\\"%ad\\\",\\\"msg\\\":\\\"%s\\\"}%n---" --date=format:'%Y-%m-%d' --name-only`;
        return new Promise((resolve, reject) => {
            child_process.exec(command, { cwd: this._repoPath }, (error, stdout, stderr) => {
                if (error) {
                    reject(error);
                } else {
                    // const commits = stdout.split('\n').map(line => {
                    //     try {
                    //         const commit = JSON.parse(line);
                    //         return commit;
                    //     } catch (e) {
                    //         return null;
                    //     }
                    // }).filter(commit => commit.hash);
                    const lines =  stdout.split('\n')
                    let commit:GitCommit|undefined = {};
                    let commits:GitCommit[] = [];
                    for (let i = 0; i < lines.length; i++) {
                        const line = lines[i];
                        if (line.startsWith('{')) {
                            try{
                                commit = JSON.parse(line);
                                if(!commit){
                                    continue
                                }
                                commit.files = [];
                            }catch(e){
                                commit = undefined
                                console.log(e);
                            }
                            
                        } else if (line === '---') {
                            if(commit){
                                commits.push(commit);
                            }
                            
                        } else if (line.trim() !== '') {
                            if(!commit){
                                continue;
                            }
                            if(!commit.files){
                                commit.files = [];
                            }
                            commit.files.push(line);
                        } else {
                            // do nothing
                            console.log('end');
                        }
                        
                    }
                    resolve(commits);
                }
            });
        });
    }

    public async getCommitFiles(commit: string): Promise<string[]> {
        const command = `git show --name-only --pretty='' ${commit}`;
        return new Promise((resolve, reject) => {
            child_process.exec(command, { cwd: this._repoPath }, (error, stdout, stderr) => {
                if (error) {
                    reject(error);
                } else {
                    const files = stdout.split('\n').filter(file => file);
                    resolve(files);
                }
            });
        });
    }
}